//------------------------------------------------------------------------------
//	FILE:			StopStart-Main.c
//
//	Description:	Basic framework for Stop/Start w/Boost 3PhIL DC/DC
//
//	Version: 		0.1
//
//  Target:  		TMS320F2803x(PiccoloB)
//
//------------------------------------------------------------------------------
//  Copyright Texas Instruments  2014
//------------------------------------------------------------------------------
//  Revision History:
//------------------------------------------------------------------------------
// 	Date		| Author	| Description
//------------------------------------------------------------------------------
// 	2014 Dec 12	| HN       	| Created new file
//------------------------------------------------------------------------------
//
// PLEASE READ - Useful notes about this Project

// Although this project is made up of several files, the most important ones are:
//	 "StopStart-Main.C"	- this file
//		- Application Initialization, Peripheral config,
//		- Application management
//		- Slower background code loops and Task scheduling
//	 "StopStart-DevInit_F2803x.C
//		- Device Initialization, e.g. Clock, PLL, WD, GPIO mapping
//		- Peripheral clock enables
//		- DevInit file will differ per each F28xxx device series, e.g. F280x, F2833x,
//	 "StopStart-ISR.asm
//		- Assembly level library Macros and any cycle critical functions are found here
//	 "StopStart-Settings.h"
//		- Global defines (settings) project selections are found here
//		- This file is referenced by both C and ASM files.

// Code is made up of sections, e.g. "FUNCTION PROTOTYPES", "VARIABLE DECLARATIONS" ,..etc
//	each section has FRAMEWORK and USER areas.
//  FRAMEWORK areas provide useful ready made "infrastructure" code which for the most part
//	does not need modification, e.g. Task scheduling, ISR call, GUI interface support,...etc
//  USER areas have functional example code which can be modified by USER to fit their appl.
//
// Code can be compiled with various build options (Incremental Builds INCR_BUILD), these
//  options are selected in file "ProjectSettings.h".  Note: "Rebuild All" compile
//  must be used if this file is modified.
//----------------------------------------------------------------------------------

#include "StopStart-Settings.h"
#include "PeripheralHeaderIncludes.h"
#include "DSP2803x_EPWM_defines.h"

#include "DPlib.h"
#include "IQmathLib.h"

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// FUNCTION PROTOTYPES
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// -------------------------------- FRAMEWORK --------------------------------------
void DeviceInit(void);
void ISR_Init(void);
void ISR_Run(void);
#ifdef FLASH
	void InitFlash();
#endif
void MemCopy();
// State Machine function prototypes
//------------------------------------
// Alpha states
void A0(void);	//state A0
void B0(void);	//state B0
void C0(void);	//state C0

// A branch states
void A1(void);	//state A1
void A2(void);	//state A2

// B branch states
void B1(void);	//state B1

// C branch states
void C1(void);	//state C1

// Variable declarations
void (*Alpha_State_Ptr)(void);	// Base States pointer
void (*A_Task_Ptr)(void);		// State pointer A branch
void (*B_Task_Ptr)(void);		// State pointer B branch
void (*C_Task_Ptr)(void);		// State pointer C branch


// ---------------------------------- USER -----------------------------------------
void ADC_SOC_CNF(int ChSel[], int Trigsel[], int ACQPS[], int IntChSel, int mode);
extern void DacDrvCnf(int16 n, int16 DACval, int16 DACsrc, int16 RAMPsrc, int16 Slope_initial);
void Boost3PhIL_PwmCnf(int16 n, int16 period, int16 sync_delay);
void PWM_DAC_TZ_CNF (int16 n, int16 period, int16 Iout_trip, int16 Vout_trip);

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// VARIABLE DECLARATIONS - GENERAL
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// -------------------------------- FRAMEWORK --------------------------------------

int16	VTimer0[4]={0,0,0,0};		// Virtual Timers slaved off CPU Timer 0 (A events)
int16	VTimer1[4]={0,0,0,0}; 		// Virtual Timers slaved off CPU Timer 1 (B events)
int16	VTimer2[4]={0,0,0,0}; 		// Virtual Timers slaved off CPU Timer 2 (C events)
int16	SerialCommsTimer=0;
int16 	CommsOKflg=0;

// Used to indirectly access all EPWM modules
volatile struct EPWM_REGS *ePWM[] =
   { &EPwm1Regs,                                              //intentional: (ePWM[0] not used)
	 &EPwm1Regs,
	 &EPwm2Regs,
	 &EPwm3Regs,
	 &EPwm4Regs,
	 #if (!DSP2802x_DEVICE_H)
 	 &EPwm5Regs,
	 &EPwm6Regs,
	 #if (DSP2803x_DEVICE_H || DSP2804x_DEVICE_H)
	 &EPwm7Regs,
	 #if (DSP2804x_DEVICE_H)
	 &EPwm8Regs
	 #endif
	 #endif
	 #endif
   };

// Used to indirectly access all Comparator modules
volatile struct COMP_REGS *Comp[] =
   { &Comp1Regs,                                               //intentional: (Comp[0] not used)
 	 &Comp1Regs,
	 &Comp2Regs,
	 #if (DSP2803x_DEVICE_H)
	 &Comp3Regs
	 #endif
   };

// Used for running BackGround in flash, and ISR in RAM
extern Uint16 *RamfuncsLoadStart, *RamfuncsLoadEnd, *RamfuncsRunStart;


// ---------------------------------- USER -----------------------------------------
// ASM module terminal pointers
// ADCDRV_8ch
extern volatile long *ADCDRV_8ch_RltPtrA;
extern volatile long *ADCDRV_8ch_RltPtrB;
extern volatile long *ADCDRV_8ch_RltPtrC;
extern volatile long *ADCDRV_8ch_RltPtrD;
extern volatile long *ADCDRV_8ch_RltPtrE;
extern volatile long *ADCDRV_8ch_RltPtrF;
extern volatile long *ADCDRV_8ch_RltPtrG;
extern volatile long *ADCDRV_8ch_RltPtrH;

// ADCDRV_1ch
extern volatile long *ADCDRV_1ch_Rlt11;

//CNTL2P2Z
extern volatile long *CNTL_2P2Z_Ref1, *CNTL_2P2Z_Out1, *CNTL_2P2Z_Fdbk1;
extern volatile long *CNTL_2P2Z_Coef1;
// Boost3PhIL_PwmDrv
extern volatile long	*Boost3PhIL_Duty1A, *Boost3PhIL_Duty1B, *Boost3PhIL_Duty1C, *Boost3PhIL_period1;

// ADC configuration variables
int		ChSel[16]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
int		TrigSel[16]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
int     ACQPS[16] =   {8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8};

volatile long	Duty = 0;					// Open-loop duty

volatile long Vref = 0;			// Output Set Voltage

int16 	Pgain1_Gui = 100, Igain1_Gui = 5, Dgain1_Gui = 0;		// PID gains for the voltage loop (values can be controlled from CCS)
int16 	b2A_Gui=0, b1A_Gui=-701, b0A_Gui=732, a2A_Gui=0, a1A_Gui=1024, a0A_Gui; // 2P2Z coefficients for the voltage loop (values can be controlled from CCS)
int16 	pid2p2z_Gui= 0, coeff_change = 1;						// Flag for switching between PID and poles and zeroes based coefficients. Flag to indicate change of coefficients
int16 	No_2p2z = 0;											// Used to disable 2P2Z execution when control loop coefficients are being changed
int16	Vout_TZ_Set = 24204;									// Initially set to 13V. Set using PWM6B as DAC. Controlled by Gui_Vout_Trip
int16	Iout_TZ_Set = 7447;										// Initially set to 10A. Set using PWM6A as DAC.Controlled by Gui_Iout_Trip.
int16	Boost_ON = 0;											// Value of 1 indicates that the system is working in 'boost' mode
int16	auto_cutoff = 1;										// Enables automated switching between 'bypass' and 'boost' modes based on input voltage

#pragma DATA_SECTION(CNTL_2P2Z_CoefStruct1, "CNTL_2P2Z_Coef");
struct CNTL_2P2Z_CoefStruct CNTL_2P2Z_CoefStruct1;
long Pgain1, Igain1, Dgain1, Dmax1;

// Variables for background support only (no need to access)
int16	i=0;					// common use incrementer
int16	temp_Scratch=0; 		// Temp here means Temporary

// History arrays are used for Running Average calculation (boxcar filter)
// Used for CCS display only, not part of control loop processing
int16	HistPtr=0;					// Index for History arrays
int16	Hist_Vin[HistorySize];
int16	Hist_Vout[HistorySize];
int16	Hist_I_fbk_A[HistorySize];
int16	Hist_I_fbk_B[HistorySize];
int16	Hist_I_fbk_C[HistorySize];
int16	Hist_Iout[HistorySize];
int16	Hist_Temp1[HistorySize];
int16	Hist_Temp2[HistorySize];
int16	Hist_Temp3[HistorySize];

volatile long Adc_Vout, Adc_Vin, Adc_Iout, Adc_I_FBK_A, Adc_I_FBK_B, Adc_I_FBK_C, Adc_Temp1, Adc_Temp2, Adc_Temp3;
long Temp_offset = 10277;
int16 Iout_offset = 0;

volatile long Vout_slew_temp = 0;		// Temp variable: used only if implementing
										// slew rate control in the slower state machine
volatile long VoutSetSlewed = 0;		// Slewed set point for the voltage loop
volatile long VoutSlewRate = 25600;		// Slew rate adjustment


//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// VARIABLE DECLARATIONS - CCS WatchWindow support
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// -------------------------------- FRAMEWORK --------------------------------------

// Variables for easier "human" use - view following variables in Watch Window as:
int16	Gui_Vset= 12288;		// Q10 - Set to 12V
int16	Gui_Vin;				// Q10
int16	Gui_Vout;				// Q10
int16	Gui_I_fbk_A;			// Q09
int16	Gui_I_fbk_B;			// Q09
int16	Gui_I_fbk_C;			// Q09
int16	Gui_Iout;				// Q09
int16	Gui_Temp1;				// Q05
int16	Gui_Temp2;				// Q05
int16	Gui_Temp3;				// Q05
int16	Gui_Vout_Trip = 16384;	// Q10 - Set to 16V
int16	Gui_Iout_Trip =	20480; 	// Q09 - Set to 40A

//Scaling Constants (values found via spreadsheet; exact value calibrated per board)
int16	iK_Vset=29789;			// 1.818 in Q14
int16	K_Vin=18022;			// 0.550 in Q15
int16	K_Vout=18022;			// 0.550 in Q15
int16	K_Iout=22528;			// 0.688 in Q15
int16	iK_Iout = 23831;		// 1.455 in Q14
int16	K_I_fbk_A =22528;		// 0.688 in Q15
int16	K_I_fbk_B =22528;		// 0.688 in Q15
int16	K_I_fbk_C =22528;		// 0.688 in Q15
int16	K_Temp1 =19200;			// 0.586 in Q15
int16	K_Temp2 =19200;			// 0.586 in Q15
int16	K_Temp3 =19200;			// 0.586 in Q15


//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// MAIN CODE - starts here
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

void main(void)
{
//=================================================================================
//	INITIALIZATION - General
//=================================================================================

//-------------------------------- FRAMEWORK --------------------------------------

	DeviceInit();	// Device Life support & GPIO

// Only used if running from FLASH
// Note that the variable FLASH is defined by the compiler
#ifdef FLASH
// Copy time critical code and Flash setup code to RAM
// The  RamfuncsLoadStart, RamfuncsLoadEnd, and RamfuncsRunStart
// symbols are created by the linker. Refer to the linker files. 
	MemCopy(&RamfuncsLoadStart, &RamfuncsLoadEnd, &RamfuncsRunStart);

// Call Flash Initialization to setup flash waitstates
// This function must reside in RAM
	InitFlash();	// Call the flash wrapper init function
#endif //(FLASH)


// Timing sync for background loops 
// Timer period definitions found in device specific PeripheralHeaderIncludes.h
	CpuTimer0Regs.PRD.all =  uSec100;	// A tasks
	CpuTimer1Regs.PRD.all =  mSec1;		// B tasks
	CpuTimer2Regs.PRD.all =  mSec1;		// C tasks

// Tasks State-machine init
	Alpha_State_Ptr = &A0;
	A_Task_Ptr = &A1;
	B_Task_Ptr = &B1;
	C_Task_Ptr = &C1;

// ---------------------------------- USER -----------------------------------------

// 2 pole / 2 Zero compensator coefficients (B2, B1, B0, A2, A1) are mapped to the
// simpler 3 coefficients P, I, D  to allow for trial & error intuitive tuning via
// CCS WatchWindow.  Note: User can modify if needed and assign full set of 5 coef.

	EALLOW;
	SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 0;
	EDIS;

// ---------------------------------- USER -----------------------------------------
#define		prd			400							// Period count = 150 KHz @ 60 MHz
#define		TZ_DAC_prd	100							// Period count = 600 KHz @ 60 MHz

#define		I_FBK_A_R	AdcResult.ADCRESULT1		//Q12
#define		VoutR		AdcResult.ADCRESULT2		//Q12
#define		VinR		AdcResult.ADCRESULT3		//Q12

#define		I_FBK_B_R	AdcResult.ADCRESULT5		//Q12
#define		IoutR		AdcResult.ADCRESULT6		//Q12
#define		Temp1R		AdcResult.ADCRESULT7		//Q12

#define		I_FBK_C_R	AdcResult.ADCRESULT9		//Q12
#define		Temp2R		AdcResult.ADCRESULT10		//Q12
#define		Temp3R		AdcResult.ADCRESULT11		//Q12


	// ADC Channel Selection
	ChSel[0] = 2;		// Dummy read
	ChSel[1] = 2;		// A2 - I_FBK_A
	ChSel[2] = 9;		// B1 - V_FBK_12VOUT
	ChSel[3] = 8;		// B0 - V_FBK_12VIN

	ChSel[4] = 4;		// Dummy read
	ChSel[5] = 4;		// A4 - I_FBK_B
	ChSel[6] = 1;		// A1 - I_FBK_12VOUT
	ChSel[7] = 10;		// B2 - TEMP_FBK_1

	ChSel[8] = 6;		// Dummy read
	ChSel[9] = 6;		// A6 - I_FBK_C
	ChSel[10] = 11;		// B3 - TEMP_FBK_2
	ChSel[11] = 12;		// B4 - TEMP_FBK_3

	// ADC Trigger Selection
	TrigSel[0] = 5;		// ePWM1, ADCSOCA
	TrigSel[1] = 5;		// ePWM1, ADCSOCA
	TrigSel[2] = 5;		// ePWM1, ADCSOCA
	TrigSel[3] = 5;		// ePWM1, ADCSOCA

	TrigSel[4] = 7;		// ePWM2, ADCSOCA
	TrigSel[5] = 7;		// ePWM2, ADCSOCA
	TrigSel[6] = 7;		// ePWM2, ADCSOCA
	TrigSel[7] = 7;		// ePWM2, ADCSOCA

	TrigSel[8] = 9;		// ePWM3, ADCSOCA
	TrigSel[9] = 9;		// ePWM3, ADCSOCA
	TrigSel[10] = 9;	// ePWM3, ADCSOCA
	TrigSel[11] = 9;	// ePWM3, ADCSOCA

	// Configure ADC
	ADC_SOC_CNF(ChSel,TrigSel,ACQPS, 16, 0);		// ACQPS=8, No ADC channel triggers an interrupt IntChSel > 15, Mode= Start/Stop (0)

	Boost3PhIL_PwmCnf(1, prd, 2);					// Configure ePWM1, ePWM2 and ePWM3

	PWM_DAC_TZ_CNF(6, TZ_DAC_prd, 90, 90);			// Configure ePWM6 for Iout and Vout trip signal generation for TZ1 (GPIO12) and TZ2 (GPIO16)

// Configure PWMs for over current and over voltage protection
	EALLOW;
// Enable TZ1 and TZ2 as one-shot source
	EPwm1Regs.DCTRIPSEL.bit.DCAHCOMPSEL = DC_TZ1; 		// DCAH = TZ1 input
	EPwm1Regs.TZDCSEL.bit.DCAEVT1 = TZ_DCAH_HI; 		// DCAEVT1 = DCAH high(will become active as TZ input goes high)
	EPwm1Regs.DCACTL.bit.EVT1SRCSEL = DC_EVT1; 			// DCAEVT1 = DCAEVT1 (not filtered)
	EPwm1Regs.DCACTL.bit.EVT1FRCSYNCSEL = DC_EVT_ASYNC;	// Take async path

	EPwm1Regs.DCTRIPSEL.bit.DCBHCOMPSEL = DC_TZ2; 		// DCBH = TZ2 input
	EPwm1Regs.TZDCSEL.bit.DCBEVT1 = TZ_DCBH_HI; 		// DCBEVT1 = DCBH high(will become active as TZ input goes high)
	EPwm1Regs.DCBCTL.bit.EVT1SRCSEL = DC_EVT1; 			// DCBEVT1 = DCBEVT1 (not filtered)
	EPwm1Regs.DCBCTL.bit.EVT1FRCSYNCSEL = DC_EVT_ASYNC;	// Take async path

	EPwm2Regs.DCTRIPSEL.bit.DCAHCOMPSEL = DC_TZ1; 		// DCAH = TZ1 input
	EPwm2Regs.TZDCSEL.bit.DCAEVT1 = TZ_DCAH_HI; 		// DCAEVT1 = DCAH high(will become active as TZ input goes high)
	EPwm2Regs.DCACTL.bit.EVT1SRCSEL = DC_EVT1; 			// DCAEVT1 = DCAEVT1 (not filtered)
	EPwm2Regs.DCACTL.bit.EVT1FRCSYNCSEL = DC_EVT_ASYNC;	// Take async path

	EPwm2Regs.DCTRIPSEL.bit.DCBHCOMPSEL = DC_TZ2; 		// DCBH = TZ2 input
	EPwm2Regs.TZDCSEL.bit.DCBEVT1 = TZ_DCBH_HI; 		// DCBEVT1 = DCBH high(will become active as TZ input goes high)
	EPwm2Regs.DCBCTL.bit.EVT1SRCSEL = DC_EVT1; 			// DCBEVT1 = DCBEVT1 (not filtered)
	EPwm2Regs.DCBCTL.bit.EVT1FRCSYNCSEL = DC_EVT_ASYNC;	// Take async path

	EPwm3Regs.DCTRIPSEL.bit.DCAHCOMPSEL = DC_TZ1; 		// DCAH = TZ1 input
	EPwm3Regs.TZDCSEL.bit.DCAEVT1 = TZ_DCAH_HI; 		// DCAEVT1 = DCAH high(will become active as TZ input goes high)
	EPwm3Regs.DCACTL.bit.EVT1SRCSEL = DC_EVT1; 			// DCAEVT1 = DCAEVT1 (not filtered)
	EPwm3Regs.DCACTL.bit.EVT1FRCSYNCSEL = DC_EVT_ASYNC;	// Take async path

	EPwm3Regs.DCTRIPSEL.bit.DCBHCOMPSEL = DC_TZ2; 		// DCBH = TZ2 input
	EPwm3Regs.TZDCSEL.bit.DCBEVT1 = TZ_DCBH_HI; 		// DCBEVT1 = DCBH high(will become active as TZ input goes high)
	EPwm3Regs.DCBCTL.bit.EVT1SRCSEL = DC_EVT1; 			// DCBEVT1 = DCBEVT1 (not filtered)
	EPwm3Regs.DCBCTL.bit.EVT1FRCSYNCSEL = DC_EVT_ASYNC;	// Take async path

// Enable DCAEVT1 & DCBEVT1 as one-shot source
	EPwm1Regs.TZSEL.bit.DCAEVT1 = 1;
	EPwm1Regs.TZSEL.bit.DCBEVT1 = 1;
	EPwm2Regs.TZSEL.bit.DCAEVT1 = 1;
	EPwm2Regs.TZSEL.bit.DCBEVT1 = 1;
	EPwm3Regs.TZSEL.bit.DCAEVT1 = 1;
	EPwm3Regs.TZSEL.bit.DCBEVT1 = 1;

// What do we want the TZ events to do?
	EPwm1Regs.TZCTL.all = 0x0FFA;						//TZA & TZB = TZ_FORCE_LO and TZ_NO_CHANGE for others
	EPwm2Regs.TZCTL.all = 0x0FFA;						//TZA & TZB = TZ_FORCE_LO and TZ_NO_CHANGE for others
	EPwm3Regs.TZCTL.all = 0x0FFA;						//TZA & TZB = TZ_FORCE_LO and TZ_NO_CHANGE for others

	EDIS;

	// Configure ePWMs to generate ADC SOC pulses
	EPwm1Regs.ETSEL.bit.SOCAEN = 1;					// Enable ePWM1 SOCA pulse
	EPwm1Regs.ETSEL.bit.SOCASEL = ET_CTRU_CMPB;		// SOCA from ePWM1 CMPBU event
	EPwm1Regs.ETPS.bit.SOCAPRD = ET_1ST;			// Trigger ePWM1 SOCA on every event

	EPwm2Regs.ETSEL.bit.SOCAEN = 1;					// Enable ePWM2 SOCA pulse
	EPwm2Regs.ETSEL.bit.SOCASEL = ET_CTRU_CMPB;		// SOCA from ePWM2 CMPBU event
	EPwm2Regs.ETPS.bit.SOCAPRD = ET_1ST;			// Trigger ePWM2 SOCA on every event

	EPwm3Regs.ETSEL.bit.SOCAEN = 1;					// Enable ePWM3 SOCA pulse
	EPwm3Regs.ETSEL.bit.SOCASEL = ET_CTRU_CMPB;		// SOCA from ePWM3 CMPBU event
	EPwm3Regs.ETPS.bit.SOCAPRD = ET_1ST;			// Trigger ePWM3 SOCA on every event

	ISR_Init();							// ASM ISR init

//====================================================================================
//	INCREMENTAL BUILD OPTIONS - NOTE: select via StopStart-Settings.h
//====================================================================================

//--------------------------------------------------------------------------------------------------------------
#if (INCR_BUILD == 1) 	// Open-Loop - Output controlled by changing Duty from watchwindow during run-time
//--------------------------------------------------------------------------------------------------------------

// Lib Module connection to "nets" 
//----------------------------------------
// ADC feedback connections
	ADCDRV_8ch_RltPtrA = &Adc_I_FBK_A;
	ADCDRV_8ch_RltPtrB = &Adc_Vout;
	ADCDRV_8ch_RltPtrC = &Adc_Vin;
	ADCDRV_8ch_RltPtrD = &Adc_I_FBK_B;
	ADCDRV_8ch_RltPtrE = &Adc_Iout;
	ADCDRV_8ch_RltPtrF = &Adc_Temp1;
	ADCDRV_8ch_RltPtrG = &Adc_I_FBK_C;
	ADCDRV_8ch_RltPtrH = &Adc_Temp2;

	ADCDRV_1ch_Rlt11 = &Adc_Temp3;

// Boost3PhIL_DRV connections
	Boost3PhIL_Duty1A = &Duty;
	Boost3PhIL_Duty1B = &Duty;
	Boost3PhIL_Duty1C = &Duty;


	Duty =_IQ24(0.0);

#endif // (INCR_BUILD == 1)


//----------------------------------------------------------------------
#if (INCR_BUILD == 2) // Closed-Loop
//----------------------------------------------------------------------

// Lib Module connection to "nets" 
//----------------------------------------
// ADC feedback connections
// ADC feedback connections
	ADCDRV_8ch_RltPtrA = &Adc_I_FBK_A;
	ADCDRV_8ch_RltPtrB = &Adc_Vout;
	ADCDRV_8ch_RltPtrC = &Adc_Vin;
	ADCDRV_8ch_RltPtrD = &Adc_I_FBK_B;
	ADCDRV_8ch_RltPtrE = &Adc_Iout;
	ADCDRV_8ch_RltPtrF = &Adc_Temp1;
	ADCDRV_8ch_RltPtrG = &Adc_I_FBK_C;
	ADCDRV_8ch_RltPtrH = &Adc_Temp2;

	ADCDRV_1ch_Rlt11 = &Adc_Temp3;

// ControlLaw2P2Z connections
	CNTL_2P2Z_Ref1 = &VoutSetSlewed;			// point to Vref
	CNTL_2P2Z_Fdbk1 = &Adc_Vout;				// point to Vout
	CNTL_2P2Z_Coef1 = &CNTL_2P2Z_CoefStruct1.b2;// point to first coeff
	CNTL_2P2Z_Out1 = &Duty;						// point to 2P2Z Uout

// Boost3PhIL_DRV connections
	Boost3PhIL_Duty1A = &Duty;
	Boost3PhIL_Duty1B = &Duty;
	Boost3PhIL_Duty1C = &Duty;

// Coefficients for the Voltage Loop
// PID coefficients & Clamping (Q26)
	Dmax1  = _IQ24(0.8);
	Pgain1 = _IQ26(0.001953);
	Igain1 = _IQ26(0.003);
	Dgain1 = _IQ26(0.0);

// Coefficient init	--- Coeeficient values in Q26
	CNTL_2P2Z_CoefStruct1.b2   = Dgain1;                           	// B2
	CNTL_2P2Z_CoefStruct1.b1   = (Igain1-Pgain1-Dgain1-Dgain1);  	// B1
	CNTL_2P2Z_CoefStruct1.b0   = (Pgain1 + Igain1 + Dgain1);      	// B0
	CNTL_2P2Z_CoefStruct1.a2   = 0.0;                              	// A2 = 0
	CNTL_2P2Z_CoefStruct1.a1   = _IQ26(1.0);                       	// A1 = 1
	CNTL_2P2Z_CoefStruct1.max  = Dmax1;					  		  	//Clamp Hi
	CNTL_2P2Z_CoefStruct1.i_min = _IQ24(-0.9); 					  	//Clamp internal Min
	CNTL_2P2Z_CoefStruct1.min  = _IQ24(0.0); 					  	//Clamp Min

	Duty =_IQ24(0.0);

#endif // (INCR_BUILD == 2)


	EALLOW;
	SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 1;
	EDIS;

	Boost_ON = 0;
	// Force trips
	EALLOW;
	EPwm1Regs.TZFRC.all = 0x0004;			//Turn-off boost phase 1
	EPwm2Regs.TZFRC.all = 0x0004;			//Turn-off boost phase 2
	EPwm3Regs.TZFRC.all = 0x0004;			//Turn-off boost phase 3
	EDIS;


//=================================================================================
//	INTERRUPT & ISR INITIALISATION (best to run this section after other initialisation)
//=================================================================================
	EALLOW;
	PieVectTable.EPWM1_INT = &ISR_Run;			// Map Interrupt
	EDIS;
	PieCtrlRegs.PIEIER3.bit.INTx1 = 1;			// PIE level enable, Grp3 / Int1

	// Configure ISR trigger
	EPwm1Regs.CMPB = 30;						// ISR trigger point
	EPwm1Regs.ETSEL.bit.INTSEL = ET_CTRU_CMPB;	// INT on CMPB-UP event
	EPwm1Regs.ETSEL.bit.INTEN = 1;				// Enable INT
	EPwm1Regs.ETPS.bit.INTPRD = ET_1ST;			// Generate ISR INT on every event

// Enable Peripheral, global Ints and higher priority real-time debug events:
	IER |= M_INT3;	// Enable CPU INT3 connected to EPWM1-6 INTs:
	EINT;   		// Enable Global interrupt INTM
	ERTM;   		// Enable Global realtime interrupt DBGM


//=================================================================================
//	BACKGROUND (BG) LOOP
//=================================================================================

//--------------------------------- FRAMEWORK -------------------------------------
	for(;;)
	{
		// State machine entry & exit point
		//===========================================================
		(*Alpha_State_Ptr)();	// jump to an Alpha state (A0,B0,...)
		//===========================================================

	}
} //END MAIN CODE


//=================================================================================
//	STATE-MACHINE SEQUENCING AND SYNCRONIZATION
//=================================================================================

//--------------------------------- FRAMEWORK -------------------------------------

void A0(void)
{
	// loop rate synchronizer for A-tasks
	if(CpuTimer0Regs.TCR.bit.TIF == 1)
	{
		CpuTimer0Regs.TCR.bit.TIF = 1;	// clear flag
		//-----------------------------------------------------------
		(*A_Task_Ptr)();		// jump to an A Task (A1,A2,A3,...)
		//-----------------------------------------------------------
		VTimer0[0]++;			// virtual timer 0, instance 0
	}
	Alpha_State_Ptr = &B0;		// Comment out to allow only A tasks
}

void B0(void)
{
	// loop rate synchronizer for B-tasks
	if(CpuTimer1Regs.TCR.bit.TIF == 1)
	{
		CpuTimer1Regs.TCR.bit.TIF = 1;	// clear flag
		//-----------------------------------------------------------
		(*B_Task_Ptr)();		// jump to a B Task (B1,B2,B3,...)
		//-----------------------------------------------------------
		VTimer1[0]++;			// virtual timer 1, instance 0
	}
	Alpha_State_Ptr = &C0;
}

void C0(void)
{
	// loop rate synchronizer for C-tasks
	if(CpuTimer2Regs.TCR.bit.TIF == 1)
	{
		CpuTimer2Regs.TCR.bit.TIF = 1;	// clear flag
		//-----------------------------------------------------------
		(*C_Task_Ptr)();		// jump to a C Task (C1,C2,C3,...)
		//-----------------------------------------------------------
		VTimer2[0]++;			// virtual timer 2, instance 0
	}
	Alpha_State_Ptr = &A0;		// Back to State A0
}


//=================================================================================
//	A - TASKS
//=================================================================================
//--------------------------------------------------------
void A1(void) // Adjust PWM trip/shutdown levels, change coefficients
//--------------------------------------------------------
{
	EALLOW;
	Vout_TZ_Set = ((long)Gui_Vout_Trip*(long)iK_Vset) >> 14;
	EPwm6Regs.CMPB = (long)Vout_TZ_Set*(long)TZ_DAC_prd>>15;

	Iout_TZ_Set = ((long)Gui_Iout_Trip*(long)iK_Iout) >> 14;
	EPwm6Regs.CMPA.half.CMPA = (long)Iout_TZ_Set*(long)TZ_DAC_prd>>15;

	EDIS;

	Pgain1 	= Pgain1_Gui*67108;		// Q26
	Igain1 	= Igain1_Gui*67108;		// Q26
	Dgain1 	= Dgain1_Gui*67108;		// Q26

  if (coeff_change == 1)
   {
		No_2p2z = 1;	// Used to disable 2P2Z execution when coefficients are being changed
	if (pid2p2z_Gui == 0)
		{
		// Voltage loop coefficient update
		CNTL_2P2Z_CoefStruct1.b2   = Dgain1;                            	// B2
    	CNTL_2P2Z_CoefStruct1.b1   = (Igain1 - Pgain1 - Dgain1 - Dgain1);  	// B1
   	 	CNTL_2P2Z_CoefStruct1.b0   = (Pgain1 + Igain1 + Dgain1);      		// B0
    	CNTL_2P2Z_CoefStruct1.a2   = 0.0;                              		// A2 = 0
    	CNTL_2P2Z_CoefStruct1.a1   = _IQ26(1.0);                       		// A1 = 1
		//    CNTL_2P2Z_CoefStruct1.max  =Dmax1;					  	 	//Clamp Hi
    	//	  CNTL_2P2Z_CoefStruct1.i_min = _IQ24(-0.9); 				  	//Clamp internal Min
		//    CNTL_2P2Z_CoefStruct1.min  =_IQ24(0.0); 					  	//Clamp Min
		}
	else
		{
		CNTL_2P2Z_CoefStruct1.b2 = b2A_Gui*65536;							// B2 - I5Q10 scaled to I5Q26
		CNTL_2P2Z_CoefStruct1.b1 = b1A_Gui*65536;							// B1
		CNTL_2P2Z_CoefStruct1.b0 = b0A_Gui*65536;							// B0
		CNTL_2P2Z_CoefStruct1.a2 = a2A_Gui*65536;							// A2
		CNTL_2P2Z_CoefStruct1.a1 = a1A_Gui*65536;							// A1
		//    CNTL_2P2Z_CoefStruct1.max  =Dmax;					  	 		//Clamp Hi
    	//	  CNTL_2P2Z_CoefStruct1.i_min = _IQ24(-0.9); 					//Clamp internal Min
		//    CNTL_2P2Z_CoefStruct1.min  =_IQ24(0.0); 					  	//Clamp Min
		}
   No_2p2z = 0;
   coeff_change = 0;
  }

	//-------------------
	A_Task_Ptr = &A2;
	//-------------------
}

//-----------------------------------------------------------------
void A2(void) // Bypass <--> Boost transition, Slew Rate ("soft-start")
//-----------------------------------------------------------------
{

 if(auto_cutoff)
 {
	if (Gui_Vin < 12083)			// If Vin < 11.8V, turn-off bypass and start boost
	{

		GpioDataRegs.GPACLEAR.bit.GPIO19 = 1;	// Disable bypass

		Boost_ON = 1;
		No_2p2z = 0;
		// Clear trips
		EALLOW;
		EPwm1Regs.TZCLR.all = 0x007F;			//TZA & TZB = TZ_FORCE_LO and TZ_NO_CHANGE for others
		EPwm2Regs.TZCLR.all = 0x007F;			//TZA & TZB = TZ_FORCE_LO and TZ_NO_CHANGE for others
		EPwm3Regs.TZCLR.all = 0x007F;			//TZA & TZB = TZ_FORCE_LO and TZ_NO_CHANGE for others
		EDIS;

//		VoutSetSlewed = 10676410;				//Set VoutSetSlewed to 11.2V
	}
	else if (Gui_Vin > 12185)		// If Vin > 11.9V, turn-on bypass and stop boost
	{
		GpioDataRegs.GPASET.bit.GPIO19 = 1;		// Enable bypass

		Boost_ON = 0;
		// Force trips
		EALLOW;
		EPwm1Regs.TZFRC.all = 0x0004;			//Turn-off boost phase 1
		EPwm2Regs.TZFRC.all = 0x0004;			//Turn-off boost phase 2
		EPwm3Regs.TZFRC.all = 0x0004;			//Turn-off boost phase 3
		EDIS;

		No_2p2z = 1;
	}
 }

// This is an example code for implementing the slew rate control in
// a slower state machine instead of implementing it in the ISR.
// VoutXSlewRate should be set as a positive value

Vout_slew_temp = Vref - VoutSetSlewed;

if (Vout_slew_temp >= VoutSlewRate) // Positive Command
{
	VoutSetSlewed = VoutSetSlewed + VoutSlewRate;
}
else
	{
	if ((-1)*(Vout_slew_temp) >= VoutSlewRate) // Negative Command
		{
		VoutSetSlewed = VoutSetSlewed - VoutSlewRate;
		}
	else	VoutSetSlewed = Vref;
	}

	//-------------------
	A_Task_Ptr = &A1;
	//-------------------
}

//=================================================================================
//	B - TASKS
//=================================================================================

//----------------------------------- USER ----------------------------------------
//----------------------------------------------------
void B1(void) // Dashboard measurements
//----------------------------------------------------
{
// Voltage measurement calculated by:
//	Gui_Vout = VoutAvg * K_Vout, where VoutAvg = sum of 8 VoutR samples

	HistPtr++;
	if (HistPtr >= HistorySize)	HistPtr = 0;

	// BoxCar Averages - Input Raw samples into BoxCar arrays
	//----------------------------------------------------------------
	Hist_Vout[HistPtr] = VoutR;
	Hist_Vin[HistPtr] = VinR;
	Hist_I_fbk_A[HistPtr] = I_FBK_A_R;
	Hist_I_fbk_B[HistPtr] = I_FBK_B_R;
	Hist_I_fbk_C[HistPtr] = I_FBK_C_R;
	Hist_Iout[HistPtr] = IoutR;
	Hist_Temp1[HistPtr] = Temp1R;
	Hist_Temp2[HistPtr] = Temp2R;
	Hist_Temp3[HistPtr] = Temp3R;

	// Voltage Measurement
	//----------------------------------------------------------------
	// view following variables in Watch Window as:
	// Gui_Vout = Q10

	temp_Scratch=0;
	for(i=0; i<HistorySize; i++)
		temp_Scratch = temp_Scratch + Hist_Vout[i];	//Q12 * 8 = Q15
	Gui_Vout = ((long)temp_Scratch*(long)K_Vout) >> 15;	//Q15*Q15 >> 15 = Q15

	temp_Scratch=0;
	for(i=0; i<HistorySize; i++)
		temp_Scratch = temp_Scratch + Hist_Vin[i];	//Q12 * 8 = Q15
	Gui_Vin = ((long)temp_Scratch*(long)K_Vin) >> 15;	//Q15*Q15 >> 15 = Q15

	temp_Scratch=0;
	for(i=0; i<HistorySize; i++)
		temp_Scratch = temp_Scratch + Hist_I_fbk_A[i];	//Q12 * 8 = Q15
	Gui_I_fbk_A = ((long)temp_Scratch*(long)K_I_fbk_A) >> 15;	//Q15*Q15 >> 15 = Q15

	temp_Scratch=0;
	for(i=0; i<HistorySize; i++)
		temp_Scratch = temp_Scratch + Hist_I_fbk_B[i];	//Q12 * 8 = Q15
	Gui_I_fbk_B = ((long)temp_Scratch*(long)K_I_fbk_B) >> 15;	//Q15*Q15 >> 15 = Q15

	temp_Scratch=0;
	for(i=0; i<HistorySize; i++)
		temp_Scratch = temp_Scratch + Hist_I_fbk_C[i];	//Q12 * 8 = Q15
	Gui_I_fbk_C = ((long)temp_Scratch*(long)K_I_fbk_C) >> 15;	//Q15*Q15 >> 15 = Q15

	temp_Scratch=0;
	for(i=0; i<HistorySize; i++)
		temp_Scratch = temp_Scratch + Hist_Iout[i];	//Q12 * 8 = Q15
	Gui_Iout = (((long)temp_Scratch + (long)Iout_offset)*(long)K_Iout) >> 15;	//Q15*Q15 >> 15 = Q15

	temp_Scratch=0;
	for(i=0; i<HistorySize; i++)
		temp_Scratch = temp_Scratch + Hist_Temp1[i];	//Q12 * 8 = Q15
	Gui_Temp1 = ((Temp_offset - (long)temp_Scratch)*(long)K_Temp1) >> 15;	//Q15*Q15 >> 15 = Q15

	temp_Scratch=0;
	for(i=0; i<HistorySize; i++)
		temp_Scratch = temp_Scratch + Hist_Temp2[i];	//Q12 * 8 = Q15
	Gui_Temp2 = ((Temp_offset - (long)temp_Scratch)*(long)K_Temp2) >> 15;	//Q15*Q15 >> 15 = Q15

	temp_Scratch=0;
	for(i=0; i<HistorySize; i++)
		temp_Scratch = temp_Scratch + Hist_Temp3[i];	//Q12 * 8 = Q15
	Gui_Temp3 = ((Temp_offset - (long)temp_Scratch)*(long)K_Temp3) >> 15;	//Q15*Q15 >> 15 = Q15


	// Voltage setting calculated by:
	// Vset = Gui_Vset * iK_Vout, where iK_Vout = 1/K_Vout (i.e. inverse K_Vout)
	// view and set following variables in Watch Window as:
	// Gui_Vset = Q10

	//Multiply with longs to get proper result then shift by 14 to turn it back into an int16
	Vref = ((long)Gui_Vset*(long)iK_Vset) >> 5; // (Q15 * Q14) >> 5 = Q24

	//-----------------
	B_Task_Ptr = &B1;	
	//-----------------
}

//=================================================================================
//	C - TASKS (Spare)
//=================================================================================

//----------------------------------------
void C1(void)
//----------------------------------------
{

	//-----------------
	C_Task_Ptr = &C1;	
	//-----------------
}


void PWM_DAC_TZ_CNF (int16 n, int16 period, int16 Iout_trip, int16 Vout_trip)
{
	ePWM[n]->TBCTL.bit.CTRMODE = TB_COUNT_UP;
	ePWM[n]->TBCTL.bit.HSPCLKDIV = TB_DIV1;
	ePWM[n]->TBCTL.bit.CLKDIV = TB_DIV1;

	ePWM[n]->TBCTL.bit.PRDLD = TB_IMMEDIATE;					// set Immediate load
	ePWM[n]->TBPRD = (period - 1);								// 'period - 1' due to up count
	ePWM[n]->TBCTR = 0;

	ePWM[n]->TBCTL.bit.PHSEN = TB_DISABLE;						// disable phase offset
	ePWM[n]->TBPHS.half.TBPHS = 0;								// no phase shift
	ePWM[n]->TBCTL.bit.SYNCOSEL = TB_CTR_ZERO; 					// sync with ePWM(n+1) "down-stream" with ePWM(n)
	ePWM[n]->TBCTL.bit.FREE_SOFT = 3;							// Free run

	ePWM[n]->CMPCTL.bit.LOADAMODE = CC_CTR_ZERO;				// load compare counter on zero event
	ePWM[n]->CMPCTL.bit.SHDWAMODE = CC_SHADOW;
	ePWM[n]->AQCTLA.bit.ZRO = AQ_SET;
	ePWM[n]->AQCTLA.bit.CAU = AQ_CLEAR;
	ePWM[n]->CMPA.half.CMPA = Iout_trip;
	ePWM[n]->AQCTLB.bit.ZRO = AQ_SET;
	ePWM[n]->AQCTLB.bit.CBU = AQ_CLEAR;
	ePWM[n]->CMPB = Vout_trip;
}


