//==============================================================================
//	Boost3PhIL_PwmCnf(int16 n, int16 period, int16 sync_delay)
//			PWM configuration for Boost DC/DC with 3 phase interleaved driver
//==============================================================================
//	FILE:			Boost3PhIL_PwmCNF.c
//
//	Description:	Config to support a 3 phase interleaved Boost driver
//
//	Version: 		0.1
//
//  Target:  		TMS320F2803x 
//------------------------------------------------------------------------------
//  Copyright Texas Instruments  2014
//------------------------------------------------------------------------------
//  Revision History:
//------------------------------------------------------------------------------
// 	Date		| Author	| Description
//------------------------------------------------------------------------------
// 	2014 Oct 08	| HN       	| Created current macro block.
//------------------------------------------------------------------------------
//				 _______________________
//  		   	|Boost3PhIL_PwmDrv n   |
//				|~~~~~~~~~~~~~~~~~~~~~~|
//	    	 -->| Duty	 	  ePWM(n)A |--> (Boost Phase 1 switch, 0 deg)
//	 	 		| 		   	    	   |
//		   	 -->| Adj	    ePWM(n+1)A |--> (Boost Phase 2 switch, 120 deg)
//	 			| 		  	     	   |
//	    		|     	    ePWM(n+2)A |--> (Boost Phase 3 switch, 240 deg)
//				|______________________|
//
//	Duty, Adj, are pointers to signal nets.
// 	Peripheral Resources used: ePWM(n)A, ePWM(n+1)A, ePWM(n+2)A, ePWM(n+3)A channels
//
// Description:
// ------------
// PWM peripheral Driver Configuration function for the following assembly macro drivers:
//	1) Boost3PhIL_PwmDrv -	Boost DC/DC with 3 phase interleaved with Adjust
//
// The function call is:
// 		Boost3PhIL_PwmCNF(int16 n, int16 period, int16 sync_delay)
//
// Function arguments defined as:
// 		n = Target master ePWM module (1, 2, 3, ...) e.g. if n=1, then master is ePWM1 with slaves ePWM2 and ePWM3
// 		period = PWM period in Sysclks
//		sync_delay = compensation for sync delay in Sysclks
//
//==================================================================================

#include "PeripheralHeaderIncludes.h"
//#include "GenericIncludes.h"			// Some Useful items
//#include "DSP280x3_EPwm.h"
#include "DSP2803x_EPWM_defines.h" 		// useful defines specific to EPWM 

extern volatile struct EPWM_REGS *ePWM[];

void Boost3PhIL_PwmCnf(int16 n, int16 period, int16 sync_delay)
{
	// Configure ePWM(n)A [Master]
	ePWM[n]->TBCTL.bit.CTRMODE = TB_COUNT_UP;
	ePWM[n]->TBCTL.bit.HSPCLKDIV = TB_DIV1;
	ePWM[n]->TBCTL.bit.CLKDIV = TB_DIV1;

	ePWM[n]->TBCTL.bit.PRDLD = TB_IMMEDIATE;					// set Immediate load
	ePWM[n]->TBPRD = (period - 1);								// 'period - 1' due to up count
	ePWM[n]->TBCTR = 0;

	ePWM[n]->TBCTL.bit.PHSEN = TB_DISABLE;						// disable phase offset
	ePWM[n]->TBPHS.half.TBPHS = 0;								// no phase shift
	ePWM[n]->TBCTL.bit.SYNCOSEL = TB_CTR_ZERO; 					// sync with ePWM(n+1) "down-stream" with ePWM(n)
	ePWM[n]->TBCTL.bit.FREE_SOFT = 3;							// Free run

	ePWM[n]->CMPCTL.bit.LOADAMODE = CC_CTR_ZERO;				// load compare counter on zero event
	ePWM[n]->CMPCTL.bit.SHDWAMODE = CC_SHADOW;
	ePWM[n]->AQCTLA.bit.ZRO = AQ_CLEAR;
	ePWM[n]->AQCTLA.bit.CAU = AQ_SET;
	ePWM[n]->CMPA.half.CMPA = 0;								// set duty 0% initially
	ePWM[n]->CMPB = 0;								// set duty 0% initially

	ePWM[n]->DBCTL.bit.OUT_MODE = DB_FULL_ENABLE;
	ePWM[n]->DBCTL.bit.POLSEL = DB_ACTV_HIC; // Active Hi Complimentary
	ePWM[n]->DBRED = 20; // initial DB value
	ePWM[n]->DBFED = 20; // initial DB value

	// Configure ePWM(n+1)A [Slave]
	ePWM[n+1]->TBCTL.bit.CTRMODE = TB_COUNT_UP;
	ePWM[n+1]->TBCTL.bit.HSPCLKDIV = TB_DIV1;
	ePWM[n+1]->TBCTL.bit.CLKDIV = TB_DIV1;

	ePWM[n+1]->TBCTL.bit.PRDLD = TB_IMMEDIATE;					// set Immediate load
	ePWM[n+1]->TBPRD = (period - 1);							// 'period - 1' due to up count
	ePWM[n+1]->TBCTR = 0;

	ePWM[n+1]->TBCTL.bit.PHSEN = TB_ENABLE;						// enable phase offset
	ePWM[n+1]->TBCTL.bit.PHSDIR = TB_UP;						// set to count up after sync
	ePWM[n+1]->TBPHS.half.TBPHS = ((period - 1)/3) + sync_delay;		// 120deg phase shift and adjust for sync delay
	ePWM[n+1]->TBCTL.bit.SYNCOSEL = TB_SYNC_IN; 				// sync with ePWM(n+2) "down-stream" with ePWM(n)
	ePWM[n+1]->TBCTL.bit.FREE_SOFT = 3;							// Free run

	ePWM[n+1]->CMPCTL.bit.LOADAMODE = CC_CTR_ZERO;				// load compare counter on zero event
	ePWM[n+1]->CMPCTL.bit.SHDWAMODE = CC_SHADOW;
	ePWM[n+1]->AQCTLA.bit.ZRO = AQ_CLEAR;
	ePWM[n+1]->AQCTLA.bit.CAU = AQ_SET;
	ePWM[n+1]->CMPA.half.CMPA = 0;								// set duty 0% initially
	ePWM[n+1]->CMPB = 0;								// set duty 0% initially


	ePWM[n+1]->DBCTL.bit.OUT_MODE = DB_FULL_ENABLE;
	ePWM[n+1]->DBCTL.bit.POLSEL = DB_ACTV_HIC; // Active Hi Complimentary
	ePWM[n+1]->DBRED = 20; // initial DB value
	ePWM[n+1]->DBFED = 20; // initial DB value

	// Configure ePWM(n+2)A [Slave]
	ePWM[n+2]->TBCTL.bit.CTRMODE = TB_COUNT_UP;
	ePWM[n+2]->TBCTL.bit.HSPCLKDIV = TB_DIV1;
	ePWM[n+2]->TBCTL.bit.CLKDIV = TB_DIV1;

	ePWM[n+2]->TBCTL.bit.PRDLD = TB_IMMEDIATE;					// set Immediate load
	ePWM[n+2]->TBPRD = (period - 1);							// 'period - 1' due to up count
	ePWM[n+2]->TBCTR = 0;

	ePWM[n+2]->TBCTL.bit.PHSEN = TB_ENABLE;						// enable phase offset
	ePWM[n+2]->TBCTL.bit.PHSDIR = TB_DOWN;						// set to count down after sync
	ePWM[n+2]->TBPHS.half.TBPHS = 2*((period - 1)/3) + sync_delay;	// 240deg phase shift and adjust for sync delay
	ePWM[n+2]->TBCTL.bit.SYNCOSEL = TB_SYNC_DISABLE; 			// sync with ePWM(n+3) "down-stream" with ePWM(n)
	ePWM[n+2]->TBCTL.bit.FREE_SOFT = 3;							// Free run

	ePWM[n+2]->CMPCTL.bit.LOADAMODE = CC_CTR_ZERO;				// load compare counter on period event
	ePWM[n+2]->CMPCTL.bit.SHDWAMODE = CC_SHADOW;
	ePWM[n+2]->AQCTLA.bit.ZRO = AQ_CLEAR;
	ePWM[n+2]->AQCTLA.bit.CAU = AQ_SET;
	ePWM[n+2]->CMPA.half.CMPA = 0;								// set duty 0% initially
	ePWM[n+2]->CMPB = 0;								// set duty 0% initially

	ePWM[n+2]->DBCTL.bit.OUT_MODE = DB_FULL_ENABLE;
	ePWM[n+2]->DBCTL.bit.POLSEL = DB_ACTV_HIC; // Active Hi Complimentary
	ePWM[n+2]->DBRED = 20; // initial DB value
	ePWM[n+2]->DBFED = 20; // initial DB value

}
