;===============================================================================
;	Boost_FeedForward n
;			FeedForward control macro module for Boost DC/DC
;===============================================================================
;	FILE:			Boost_FeedForward.asm
;
;	Description:	FeedForward control macro for Boost DC/DC
;
;	Version: 		0.1
;
;   Target:  		TMS320F2803x
;-------------------------------------------------------------------------------
;  	Copyright Texas Instruments  2009
;-------------------------------------------------------------------------------
; 	Revision History:
;-------------------------------------------------------------------------------
; 	Date		| Author	| Description
;-------------------------------------------------------------------------------
; 	2009 Dec 15	| DC       	| Created current macro block.
;-------------------------------------------------------------------------------
;			 	 ______________________
;				|  Boost_FeedForward  |
;				|~~~~~~~~~~~~~~~~~~~~~|
;		 	 -->| Nom			U_Out |-->
;		 	 -->| Fdbk			Debug |-->
;		 	 -->| U_Max               |
;		 		|_____________________|
;
;	Nom, Fdbk, U_Max, U_Out, Debug: pointers to Signal Nets
;
; Description:
; ------------
; Performs FeedForward control U_Out = 1 - Fdbk*(1/Nom)
;===============================================================================

;===============================
Boost_FeedForward_INIT	.macro n
;===============================
; Constant Declarations
;-------------------------------------------------------------------------------
;U_Out saturation
UMIN		.set	0x0000	; 0

; Scaling factor to make Gain=1 when Nom==Fdbk (due to ADC feedback gain differences)
ADC_RATIO_H	.set	0x03FF	; upper 16bits of 1 in Q26
ADC_RATIO_L	.set	0xFFFF	; lower 16bits of 1 in Q26

; One in various Q-values
ONE20_H		.set	0x000F	; upper 16bits of 1 in Q20
ONE20_L		.set	0xFFFF	; lower 16bits of 1 in Q20
ONE31_H		.set	0x7FFF	; upper 16bits of 1 in Q31
ONE31_L		.set	0xFFFF	; lower 16bits of 1 in Q31

; Variable Declarations
;-------------------------------------------------------------------------------
_BOOST_FEEDFORWARD_Nom:n:		.usect "BOOST_FEEDFORWARD_data",2,1,1	; Input Terminal 1
_BOOST_FEEDFORWARD_Fdbk:n:		.usect "BOOST_FEEDFORWARD_data",2		; Input Terminal 2
_BOOST_FEEDFORWARD_U_Max:n:		.usect "BOOST_FEEDFORWARD_data",2 		; Input Terminal 3
_BOOST_FEEDFORWARD_U_Out:n:		.usect "BOOST_FEEDFORWARD_data",2 		; Output Terminal 1
bff_min_u_out:n:				.usect "BOOST_FEEDFORWARD_data",2		; "Local" memory Long
bff_u_out:n:					.usect "BOOST_FEEDFORWARD_data",2		; "Local" memory Long
bff_tempvar:n:					.usect "BOOST_FEEDFORWARD_data",2		; "Local" memory Long
bff_adc_ratio:n:				.usect "BOOST_FEEDFORWARD_data",2		; "Local" memory Long
bff_one_q20:n:					.usect "BOOST_FEEDFORWARD_data",2		; "Local" memory Long
bff_one_q31:n:					.usect "BOOST_FEEDFORWARD_data",2		; "Local" memory Long
; Publish Terminal Pointers for access from the C environment (optional)
;-------------------------------------------------------------------------------
					.def 	_BOOST_FEEDFORWARD_Nom:n:
					.def 	_BOOST_FEEDFORWARD_Fdbk:n:
					.def	_BOOST_FEEDFORWARD_U_Max:n:
					.def 	_BOOST_FEEDFORWARD_U_Out:n:

		; set input terminals to point to ZeroNet
			MOVL	XAR2, #ZeroNet
			MOVW	DP, #_BOOST_FEEDFORWARD_Nom:n:
			MOVL	@_BOOST_FEEDFORWARD_Nom:n:, XAR2
			MOVL	@_BOOST_FEEDFORWARD_Fdbk:n:, XAR2
			MOVL	@_BOOST_FEEDFORWARD_U_Max:n:, XAR2
		; set output terminals to point to DummyNet
			MOVL	XAR2, #DummyNet
			MOVL	@_BOOST_FEEDFORWARD_U_Out:n:, XAR2

		; save local variables
			MOV		@bff_min_u_out:n:+1, 	#UMIN
			MOV		@bff_min_u_out:n:, 		#UMIN
			MOV		@bff_adc_ratio:n:+1, 	#ADC_RATIO_H
			MOV		@bff_adc_ratio:n:, 		#ADC_RATIO_L
			MOV		@bff_one_q20:n:+1, 		#ONE20_H
			MOV		@bff_one_q20:n:, 		#ONE20_L
			MOV		@bff_one_q31:n:+1, 		#ONE31_H
			MOV		@bff_one_q31:n:, 		#ONE31_L
						.endm

;=======================
Boost_FeedForward	.macro n
;=======================

	; Set up address pointers:
		MOVW	DP, #_BOOST_FEEDFORWARD_Nom:n:
		MOVL 	XAR0, @_BOOST_FEEDFORWARD_Nom:n:	; Net pointer to Nom (XAR0)
		MOVL    XAR1, @_BOOST_FEEDFORWARD_Fdbk:n:	; Net pointer to Fdbk (XAR1)
		MOVL	XAR3, @_BOOST_FEEDFORWARD_U_Max:n:	; Net pointer to U_Max (XAR3)
		MOVL    XAR4, @_BOOST_FEEDFORWARD_U_Out:n:	; Net pointer to U_Out (XAR4)

	; Check if Nom > Fdbk
		MOV		AL, *XAR0						; ACC = Nom						Q15
		CMP		AL, *XAR1						; Nom - Fdbk
		B		bff_valid, GT					; Nom > Fdbk, jump to FF calculations
		MOVB	ACC, #0							; Nom <= Fdbk, set U_out to 0
		LB		bff_u_out_sat					; Jump to U_Out saturation

bff_valid:
	; Calculate 1/Nom
		MOVU	ACC, *XAR0					
		MOVL	@bff_tempvar:n:, ACC			; Load Den with Nom				Q15
		MOVL	P, @bff_one_q31:n:				; Load Num with 1				Q31
		MOVB	ACC, #0
		RPT		#31
	  ||SUBCUL	ACC, @bff_tempvar:n:			; P = 1/Nom						Q31/Q15=Q16
		MOVL	ACC, P
		LSL		ACC, #10
		MOVL	@bff_u_out:n:, ACC				; bff_u_out = 1/Nom				Q26

	; Calculate Fdbk/Nom = Fdbk * 1/Nom
		MOVU	ACC, *XAR1						; ACC = Fdbk					Q15
		LSL		ACC, #11						; ACC = Fdbk<<11				Q15<<11=Q26
		MOVL	@bff_tempvar:n:, ACC			; bff_tempvar = Fdbk			Q26
		MOVL	XT, @bff_u_out:n:				; XT = 1/Nom					Q26
		QMPYL	ACC, XT, @bff_tempvar:n:		; ACC = 1/Nom * Fdbk			Q26*Q26=Q52-Q32=Q20
		LSL		ACC, #6							; convert to Q26
		MOVL	@bff_u_out:n:, ACC				; bff_u_out = Fdbk/Nom			Q26
		
	; Normalize Fdbk/Nom by adc_ratio
		MOVL	XT, @bff_u_out:n:				; XT = Fdbk/Nom					Q26
		QMPYL	ACC, XT, @bff_adc_ratio:n:		; ACC = Fdbk/Nom * ADC_Ratio	Q26*Q26=Q52-Q32=Q20
		MOVL	@bff_u_out:n:, ACC				; bff_u_out= Fdbk/Nom*ADC_Ratio	Q20

	; Calculate u_out = (1-Fdbk/Nom*adc_ratio)
		MOVL	ACC, @bff_one_q20:n:			; ACC = 1						Q20
		SUBL	ACC, @bff_u_out:n:				; ACC = 1-Fdbk/Nom*ADC_Ratio	Q20
;		MOVL	@bff_u_out:n:, ACC				; bff_u_out= Fdbk/Nom*ADC_Ratio	Q20

bff_u_out_sat:
	; Saturate u_out
;		MOVL	ACC, @bff_u_out:n:				; ACC = 1-Fdbk/Nom*ADC_Ratio	Q20
;		MINL 	ACC, *XAR3						; Saturate to <= Dmax	 		Q20
;		MAXL 	ACC, @bff_min_u_out:n:			; Saturate to >= Dmin
 
	; Output final U_Out result
		LSL		ACC, #11						; Convert from Q20 to Q31
		MOV		*XAR4, AH						; Save U_Out					Q15
					.endm  
